/*
 * glv.cpp
 *
 *  Created on: Jul 14, 2009
 *      Author: bh
 */

#include "glv.h"
#include "glv.moc"

#include "videoFileDialog.h"


Glv::Glv ( QWidget *parent): QMainWindow ( parent ), is(NULL), refreshTimingTimer(0)
{
    setupUi ( this );

    QObject::connect(actionQuit, SIGNAL(activated()), this, SLOT(close()));

    vcontrolDockWidget->setEnabled(false);
    vinfoDockWidget->setEnabled(false);
    vconfigDockWidget->setEnabled(false);

    refreshTimingTimer = new QTimer(this);
    Q_CHECK_PTR(refreshTimingTimer);
    refreshTimingTimer->setInterval(200);
    QObject::connect(refreshTimingTimer, SIGNAL(timeout()), this, SLOT(refreshTiming()));
    QObject::connect(vcontrolDockWidget, SIGNAL(visibilityChanged(bool)), this, SLOT(updateRefreshTimerState()));

//    timeLineEdit->setInputMask("00:00:09:00; ");

    setWindowTitle(tr("BHBN glv"));
}

Glv::~Glv() {

    delete refreshTimingTimer;

    if (is) {
        delete is;
    }

}

void Glv::updateRefreshTimerState(){

    if (is && !is->isPaused() && is->isRunning() && vcontrolDockWidget->isVisible())
        refreshTimingTimer->start();
    else
        refreshTimingTimer->stop();

}

void Glv::on_actionFormats_and_Codecs_activated(){

    VideoFile::displayFormatsCodecsInformation();

}

void Glv::on_actionOpenGL_extensions_activated(){

    videoFileDisplayWidget::showGlExtensionsInformationDialog();

}


void Glv::displayLogMessage(QString msg){

//    QMessageBox::information(this, "glv Log", msg, QMessageBox::Ok, QMessageBox::Ok);
    qDebug("Log %s", msg.toLatin1().data());

}

void Glv::displayErrorMessage(QString msg){

    QMessageBox::critical(this, "glv error", msg, QMessageBox::Ok, QMessageBox::Ok);

}

void Glv::on_actionOpen_activated(){
    QString fileName = QString();
    static QDir d = QDir::home();


#ifndef NO_VIDEO_FILE_DIALOG_PREVIEW
    videoFileDialog *mfd = new videoFileDialog(this, "Open a video or a picture", d.absolutePath());

    QObject::connect(mfd, SIGNAL(error(QString)), this, SLOT(displayErrorMessage(QString)));

    QStringList fileNames;
    if (mfd->exec())
        fileNames = mfd->selectedFiles();

    if (!fileNames.empty())
        fileName = fileNames.first();

//    mfd->filterSelected()

#else
    fileName = QFileDialog::getOpenFileName(this, tr("Open File"),
                                                    d.absolutePath(),
                                                    tr("Video (*.mov *.avi *.wmv *.mpeg *.mp4 *.mpg *.vob *.swf *.flv);;Image (*.png *.jpg *.jpeg *.tif *.tiff *.gif *.tga *.sgi *.bmp)"));
#endif

//    qDebug("Glv fileName = %s", fileName.toLatin1().data());
//    fileName = "/home/bh/Videos/Mess/windspeedEsbjerg.avi";

    d.setPath(fileName);

    if ( !fileName.isEmpty() && QFileInfo(fileName).isFile() ) {

        if (is) {
            delete is;
        }
        refreshTimingTimer->stop();

        if ( !mfd->customSizeChecked() && (videoFileDisplayWidget::glSupportsExtension("GL_EXT_texture_non_power_of_two") || videoFileDisplayWidget::glSupportsExtension("GL_ARB_texture_non_power_of_two") ) )
            is = new VideoFile(this);
        else
        {
            is = new VideoFile(this, true, SWS_FAST_BILINEAR, PIX_FMT_RGB24);
        }

        // forward error messages to display fn
        QObject::connect(is, SIGNAL(error(QString)), this, SLOT(displayErrorMessage(QString)));

        // sets the video source for the video display widget
        videowidget->setVideo(is);

        // CONTROL signals from GUI to VideoFile
        QObject::connect(startButton, SIGNAL(toggled(bool)), is, SLOT(play(bool)));
        QObject::connect(pauseButton, SIGNAL(toggled(bool)), is, SLOT(pause(bool)));
        QObject::connect(seekBackwardButton, SIGNAL(clicked()), is, SLOT(seekBackward()));
        QObject::connect(seekForwardButton, SIGNAL(clicked()), is, SLOT(seekForward()));
        QObject::connect(seekBeginButton, SIGNAL(clicked()), is, SLOT(seekBegin()));
        QObject::connect(playSpeedBox, SIGNAL(currentIndexChanged(int)), is, SLOT(setPlaySpeed(int)));
        QObject::connect(videoLoopButton, SIGNAL(toggled(bool)), is, SLOT(setLoop(bool)));
        QObject::connect(markInButton, SIGNAL(clicked()), is, SLOT(setMarkIn()));
        QObject::connect(markOutButton, SIGNAL(clicked()), is, SLOT(setMarkOut()));
        QObject::connect(resetMarkInButton, SIGNAL(clicked()), is, SLOT(resetMarkIn()));
        QObject::connect(resetMarkOutButton, SIGNAL(clicked()), is, SLOT(resetMarkOut()));
        QObject::connect(dirtySeekCheckBox, SIGNAL(toggled(bool)), is, SLOT(setOptionAllowDirtySeek(bool)));
        QObject::connect(resetToBlackCheckBox, SIGNAL(toggled(bool)), is, SLOT(setOptionRevertToBlackWhenStop(bool)));
        QObject::connect(restartWhereStoppedCheckBox, SIGNAL(toggled(bool)), is, SLOT(setOptionRestartToMarkIn(bool)));

        // DISPLAY consistency from VideoFile to GUI
        QObject::connect(is, SIGNAL(paused(bool)), pauseButton, SLOT(setChecked(bool)));
        QObject::connect(is, SIGNAL(running(bool)), startButton, SLOT(setChecked(bool)));
        QObject::connect(is, SIGNAL(running(bool)), videoControlFrame, SLOT(setEnabled(bool)));
        QObject::connect(is, SIGNAL(running(bool)), speedControlFrame, SLOT(setEnabled(bool)));
        QObject::connect(is, SIGNAL(running(bool)), timingControlFrame, SLOT(setEnabled(bool)));
        QObject::connect(is, SIGNAL(info(QString)), statusbar, SLOT(showMessage(QString)));

        // Consistency and update timer control from VideoFile
        QObject::connect(is, SIGNAL(markingChanged()), this, SLOT(updateMarks()));
        QObject::connect(is, SIGNAL(running(bool)), this, SLOT(updateRefreshTimerState()));
        QObject::connect(is, SIGNAL(paused(bool)), this, SLOT(updateRefreshTimerState()));

        // can we open the file ?
        if ( is->open(fileName) ) {
            vinfoDockWidget->setEnabled(true);
            FileNameLineEdit->setText(fileName);
            CodecNameLineEdit->setText(is->getCodecName());
            // display size
            if (mfd->customSizeChecked()) {
                widthLineEdit->setText( QString("%1 (%2)").arg(is->getStreamFrameWidth()).arg(is->getFrameWidth()));
                heightLineEdit->setText( QString("%1 (%2)").arg(is->getStreamFrameHeight()).arg(is->getFrameHeight()));
            } else {
                widthLineEdit->setText( QString("%1").arg(is->getFrameWidth()));
                heightLineEdit->setText( QString("%1").arg(is->getFrameHeight()));
            }
            framerateLineEdit->setText(QString().setNum(is->getFrameRate(),'f',2));
            videoLoopButton->setChecked(is->isLoop());

            // is there more than one frame ?
            if ( is->getEnd() > 1 ) {
                // yes, its a video, we can control it
                vcontrolDockWidget->setEnabled(true);
                vconfigDockWidget->setEnabled(true);
                // display times
                on_actionShow_frames_toggled(actionShow_frames->isChecked());

                // restore config
                is->setOptionRestartToMarkIn( restartWhereStoppedCheckBox->isChecked());
                is->setOptionRevertToBlackWhenStop( resetToBlackCheckBox->isChecked());
                is->setOptionAllowDirtySeek(dirtySeekCheckBox->isChecked());

                // only the speed is not connected to signals:
                playSpeedBox->setCurrentIndex(3);

                startButton->setEnabled( true );
            } else {
                // no, its a picture, we can't control it
                vcontrolDockWidget->setEnabled(false);
                vconfigDockWidget->setEnabled(false);
                // show some info even if one frame only
                endLineEdit->setText( QString().setNum(is->getEnd()) );
                timeLineEdit->setText( QString().setNum(is->getBegin()) );
                markInLineEdit->setText( QString().setNum( is->getMarkIn() ));
                markOutLineEdit->setText( QString().setNum( is->getMarkOut() ));
            }
        }
    }


#ifndef NO_VIDEO_FILE_DIALOG_PREVIEW
    delete mfd;
#endif

}

void Glv::on_actionShow_frames_toggled(bool on){

    if (on){
        endLineEdit->setText( QString().setNum(is->getEnd()) );
        timeLineEdit->setText( QString().setNum(is->getBegin()) );
    } else {
        endLineEdit->setText( is->getTimeFromFrame(is->getEnd()) );
        timeLineEdit->setText( is->getTimeFromFrame(is->getBegin()) );
    }
    updateMarks();
}

void Glv::on_markInSlider_sliderReleased (){

    double percent = (double)(markInSlider->value())/1000.0;
    int64_t pos = (int64_t) ( is->getEnd()  * percent );
    pos += is->getBegin();
    is->setMarkIn(pos);

}

void Glv::on_markOutSlider_sliderReleased (){

    double percent = (double)(markOutSlider->value())/1000.0;
    int64_t pos = (int64_t) ( is->getEnd()  * percent );
    pos += is->getBegin();
    is->setMarkOut(pos);

}

void Glv::refreshTiming(){

    if (!skipNextRefresh) {

        int f_percent = (int) ( (double)( is->getCurrentFrameTime() - is->getBegin() ) / (double)( is->getEnd() - is->getBegin() ) * 1000.0) ;
        frameSlider->setValue(f_percent);

        if (actionShow_frames->isChecked())
            timeLineEdit->setText( QString().setNum(is->getCurrentFrameTime()) );
        else
            timeLineEdit->setText( is->getTimeFromFrame(is->getCurrentFrameTime()) );
    }
    else
        skipNextRefresh = false;

}



void  Glv::on_frameSlider_sliderPressed (){

    // do not update slider position automatically anymore ; this interferes with user input
    refreshTimingTimer->stop();
}

void  Glv::on_frameSlider_sliderReleased (){

    // slider moved, frame was displayed, still paused; we un-pause if it was playing.
    is->pause(pauseButton->isChecked());

    // restart the refresh timer if it should be
    updateRefreshTimerState();
}

void Glv::on_frameSlider_sliderMoved (int v){

    // disconnect the button from the VideoFile signal ; this way when we'll unpause bellow, the button will keep its state
    QObject::disconnect(is, SIGNAL(paused(bool)), pauseButton, SLOT(setChecked(bool)));

    // the trick; call a method when the frame will be ready!
    QObject::connect(is, SIGNAL(frameReady(int)), this, SLOT(pauseAfterFrame()));

    // compute where we should jump to
    double percent = (double)(v)/ (double)frameSlider->maximum();
    int64_t pos = (int64_t) ( is->getEnd()  * percent );
    pos += is->getBegin();

    // request seek ; we need to have the VideoFile process running to go there
    is->seekToPosition(pos);

    // let the VideoFile run till it displays the frame seeked
    is->pause(false);

    // cosmetics to show the time of the frame (refreshTiming disabled)
    if (actionShow_frames->isChecked())
        timeLineEdit->setText( QString().setNum(pos) );
    else
        timeLineEdit->setText( is->getTimeFromFrame(pos) );
}


void Glv::on_frameSlider_actionTriggered (int a) {


    switch (a) {
        case QAbstractSlider::SliderMove: // wheel
        case QAbstractSlider::SliderSingleStepAdd :
        case QAbstractSlider::SliderSingleStepSub :
        case QAbstractSlider::SliderPageStepAdd : // clic forward
        case QAbstractSlider::SliderPageStepSub : // clic backward
            // avoid jump in slider (due to timer refresh of the slider position from movie values)
            skipNextRefresh = true;

            if (pauseButton->isChecked())
                on_frameSlider_sliderMoved (frameSlider->sliderPosition ());
            else{
                // compute where we should jump to
                double percent = (double)(frameSlider->sliderPosition ())/ (double)frameSlider->maximum();
                int64_t pos = (int64_t) ( is->getEnd()  * percent );
                pos += is->getBegin();

                // request seek ; we need to have the VideoFile process running to go there
                is->seekToPosition(pos);
            }
            // avoid jump in slider (due to timer refresh of the slider position from movie values)
            skipNextRefresh = true;
            break;
    }

}


void Glv::pauseAfterFrame (){

    is->pause(true);

    // do not keep calling pause method for each frame !
    QObject::disconnect(is, SIGNAL(frameReady(int)), this, SLOT(pauseAfterFrame()));
    // reconnect the pause button
    QObject::connect(is, SIGNAL(paused(bool)), pauseButton, SLOT(setChecked(bool)));

}


void Glv::updateMarks (){

    int i_percent = (int) ( (double)( is->getMarkIn() - is->getBegin() ) / (double)( is->getEnd() - is->getBegin() ) * 1000.0) ;
    int o_percent = (int) ( (double)( is->getMarkOut() - is->getBegin() ) / (double)( is->getEnd() - is->getBegin() ) * 1000.0) ;

    markInSlider->setValue(i_percent);
    markOutSlider->setValue(o_percent);

    if (actionShow_frames->isChecked()) {
        markInLineEdit->setText( QString().setNum( is->getMarkIn() ));
        markOutLineEdit->setText( QString().setNum( is->getMarkOut() ));
    } else {
        markInLineEdit->setText( is->getTimeFromFrame( is->getMarkIn() ));
        markOutLineEdit->setText( is->getTimeFromFrame( is->getMarkOut() ));
    }

}


