

//#include "glwidget.h"
#include "videoFileDisplayWidget.moc"

#include <QtGui/QImage>
#include <QFileDialog>
#include <QMessageBox>

#define UNIT 1.0

QStringList videoFileDisplayWidget::listofextensions;

videoFileDisplayWidget::videoFileDisplayWidget(QWidget *parent)
  : QGLWidget(QGLFormat(QGL::AlphaChannel), parent), is(NULL), qList(0), cubeTexture(0)

{
    if (!format().depth())
        qWarning("Could not get depth buffer; results will be suboptimal");
    if (!format().rgba())
        qWarning("Could not set rgba buffer; results will be suboptimal");
    if (!format().directRendering())
        qWarning("Could not set direct rendering; results will be suboptimal");
    if (!format().doubleBuffer())
        qWarning("Could not set double buffering; results will be suboptimal");


    if (listofextensions.isEmpty()) {
        makeCurrent();
        QString allextensions = QString( (char *) glGetString(GL_EXTENSIONS));
        listofextensions = allextensions.split(" ", QString::SkipEmptyParts);
    }

}

videoFileDisplayWidget::~videoFileDisplayWidget()
{
    if (qList)
        glDeleteLists(qList, 1);
}


void videoFileDisplayWidget::setVideo(VideoFile *f){

    is = f;
    makeCurrent();

    if (is) {
        glEnable(GL_TEXTURE_2D);
        QObject::connect(is, SIGNAL(frameReady(int)), this, SLOT(updateFrame(int)));
    }
    else {
        glDisable(GL_TEXTURE_2D);
        update();
    }
}



void videoFileDisplayWidget::initializeGL()
{
    // Enables smooth color shading
    glShadeModel(GL_FLAT);

    // disable depth and lighting by default
    glDisable(GL_DEPTH_TEST);
    glDisable(GL_LIGHTING);

    // Enables texturing
    glEnable(GL_TEXTURE_2D);
    // Pure texture color (no lighting)
    glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE);

    // Turn blending on
    glEnable(GL_BLEND);

    // Blending Function For transparency Based On Source Alpha Value
//    glBlendFunc(GL_SRC_ALPHA, GL_ONE);
    glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);

    // ANTIALIASING
    glEnable(GL_LINE_SMOOTH);
    glHint(GL_LINE_SMOOTH_HINT, GL_NICEST);

     // setup color
     glClearColor(0.0, 0.0, 0.0, 1.0f);
//     qglClearColor(QColor::fromRgb(1.0, 0.0, 0.0).dark());

//    cubeTexture = bindTexture(QImage("./cubelogo.png"));
    glGenTextures(1, &cubeTexture);

    glBindTexture(GL_TEXTURE_2D, cubeTexture);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
//    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
//    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);


    qList = glGenLists(1);
    glNewList(qList, GL_COMPILE);
    {
        qglColor(QColor::fromRgb(100, 100, 100).lighter());
        glBegin(GL_QUADS); // begin drawing a square

        // Front Face (note that the texture's corners have to match the quad's corners)
        glNormal3f(0.0f, 0.0f, 1.0f); // front face points out of the screen on z.
        glTexCoord2f(0.0f, 1.0f);
        glVertex3f(-1.0f, -1.0f, 0.0f); // Bottom Left
        glTexCoord2f(1.0f, 1.0f);
        glVertex3f(1.0f, -1.0f, 0.0f); // Bottom Right
        glTexCoord2f(1.0f, 0.0f);
        glVertex3f(1.0f, 1.0f, 0.0f); // Top Right
        glTexCoord2f(0.0f, 0.0f);
        glVertex3f(-1.0f, 1.0f, 0.0f); // Top Left

        glEnd();
    }
    glEndList();

}

void videoFileDisplayWidget::resizeGL(int w, int h)
{
    glViewport(0, 0, w, h);

    // Setup specific projection and view for this window
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
    gluOrtho2D(-UNIT, UNIT, -UNIT, UNIT);

    glMatrixMode(GL_MODELVIEW);
}

void videoFileDisplayWidget::updateFrame (int i)
{

    makeCurrent();

    // start update texture
    if (is) {
        glBindTexture(GL_TEXTURE_2D, cubeTexture);
        const VideoPicture *vp = is->getPictureAtIndex(i);
        if (vp && vp->isAllocated()) {

            vp->saveToPPM();

            if ( vp->getFormat() == PIX_FMT_RGBA)
                glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA,  vp->getWidth(),
                         vp->getHeight(), 0, GL_RGBA, GL_UNSIGNED_BYTE,
                         vp->getBuffer() );
            else
                glTexImage2D(GL_TEXTURE_2D, 0, GL_RGB,  vp->getWidth(),
                         vp->getHeight(), 0, GL_RGB, GL_UNSIGNED_BYTE,
                         vp->getBuffer() );

        }
    }
    // end

    update();
}

void videoFileDisplayWidget::paintGL()
{
    glClear(GL_COLOR_BUFFER_BIT);

    glCallList(qList);
}

bool videoFileDisplayWidget::glSupportsExtension(QString extname) {

    if (listofextensions.isEmpty()) {
        videoFileDisplayWidget *tmp = new videoFileDisplayWidget();
        delete tmp;
    }

    return listofextensions.contains(extname, Qt::CaseInsensitive);
}



void videoFileDisplayWidget::showGlExtensionsInformationDialog(){

    QDialog *openglExtensionsDialog;
    QVBoxLayout *verticalLayout;
    QLabel *label;
    QListView *extensionsListView;
    QDialogButtonBox *buttonBox;

    openglExtensionsDialog = new QDialog(0);
    openglExtensionsDialog->resize(442, 358);
    verticalLayout = new QVBoxLayout(openglExtensionsDialog);
    label = new QLabel(openglExtensionsDialog);

    verticalLayout->addWidget(label);

    extensionsListView = new QListView(openglExtensionsDialog);

    verticalLayout->addWidget(extensionsListView);

    buttonBox = new QDialogButtonBox(openglExtensionsDialog);
    buttonBox->setOrientation(Qt::Horizontal);
    buttonBox->setStandardButtons(QDialogButtonBox::Ok);

    verticalLayout->addWidget(buttonBox);

    openglExtensionsDialog->setWindowTitle(tr("OpenGL Extensions"));
    label->setText(tr("Supported OpenGL extensions:"));

    if (listofextensions.isEmpty()) {
        videoFileDisplayWidget *tmp = new videoFileDisplayWidget();
        delete tmp;
    }
    QAbstractItemModel *model = new QStringListModel(listofextensions);
    extensionsListView->setModel(model);

    QObject::connect(buttonBox, SIGNAL(accepted()), openglExtensionsDialog, SLOT(accept()));
    QObject::connect(buttonBox, SIGNAL(rejected()), openglExtensionsDialog, SLOT(reject()));

    openglExtensionsDialog->exec();

    delete verticalLayout;
    delete label;
    delete extensionsListView;
    delete buttonBox;
    delete model;
    delete openglExtensionsDialog;
}

